"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// https://github.com/caffeinalab/preferences/blob/master/index.js
const crypto = require("crypto");
const fs = require("fs");
const mkdirp = require("mkdirp");
const os_1 = require("os");
const path = require("path");
const writeFileAtomic = require("write-file-atomic");
function Preferences(id, dirName, defs, options = { key: null }) {
    const preferences = {};
    const identifier = id.replace(/[\/\?<>\\:\*\|" :]/g, '.').replace(/\.+/g, '.');
    // const path = require('path')
    const homeDir = os_1.homedir();
    const dirpath = path.join(homeDir, '.config', dirName || 'composer');
    const filepath = path.join(dirpath, identifier + '.pref');
    const password = (() => {
        const key = options.key || path.join(homeDir, '.ssh', 'id_rsa');
        try {
            // Use private SSH key or...
            return fs.readFileSync(key).toString('utf8');
        }
        catch (e) {
            // ...fallback to an id dependant password
            return 'PREFS-' + identifier;
        }
    })();
    let savePristine = false;
    let savedData = null;
    const algo = 'aes-256-cbc';
    const hash = crypto.createHash('sha256').update(String(password)).digest('base64');
    const iv16 = hash.substr(0, 16);
    const key32 = hash.substr(0, 32);
    function encode(text) {
        const cipher = crypto.createCipheriv(algo, key32, iv16);
        return cipher.update(Buffer.from(text).toString('utf8'), 'utf8', 'hex') + cipher.final('hex');
    }
    function decode(text) {
        const decipher = crypto.createDecipheriv(algo, key32, iv16);
        return decipher.update(String(text), 'hex', 'utf8') + decipher.final('utf8');
    }
    function decodeUsingDeprecatedDecipher(text) {
        const decipher = crypto.createDecipher('aes128', password);
        return decipher.update(String(text), 'hex', 'utf8') + decipher.final('utf8');
    }
    function save() {
        const payload = encode(String(JSON.stringify(preferences) || '{}'));
        try {
            mkdirp.sync(dirpath, 0o700);
            writeFileAtomic.sync(filepath, payload, {
                mode: 0o600,
            });
        }
        catch (err) {
            console.log(err);
        }
    }
    try {
        // Try to read and decode preferences saved on disc
        savedData = JSON.parse(decode(fs.readFileSync(filepath, 'utf8')));
    }
    catch (err) {
        try {
            savedData = JSON.parse(decodeUsingDeprecatedDecipher(fs.readFileSync(filepath, 'utf8')));
            console.log('Used old decoding approach. The preferences will be auto-encoded with new cipher.');
        }
        catch (_a) {
            // Read error (maybe file doesn't exist) so update with defaults
            savedData = defs || {};
            savePristine = true;
        }
    }
    // Clone object
    for (const o in savedData) {
        if (savedData.hasOwnProperty(o)) {
            preferences[o] = savedData[o];
        }
    }
    // Config file was empty, save default values
    if (savePristine) {
        save();
    }
    // Save all on program exit
    process.on('exit', save);
    return savedData;
}
exports.Preferences = Preferences;
